#ifndef __ABSTRACT_CPP__

#define __ABSTRACT_CPP__


#include "include.cpp"


#define CentreX 400
#define CentreY 300


//************************     Abstraction Layer    ****************************

Display *my_Display;
GC my_GC;
Colormap cmap;
XColor fgcolor, bgcolor;
Window my_Window;
Visual *default_visual;
Status status;
Cursor watch_cursor;


unsigned int display_width, display_height;
unsigned int width, height;
int screen_number;

//Prototype Declaration
void create_simple_window (int, int, unsigned int, unsigned int);
void create_gc ();

int
initialize (char *hostname = "")
{

  my_Display = XOpenDisplay (hostname);
  if (my_Display == NULL)
    {
      printf ("It is a display Error !!!");
      return 1;
    }

  screen_number = DefaultScreen (my_Display);
  display_width = DisplayWidth (my_Display, screen_number);
  display_height = DisplayHeight (my_Display, screen_number);

  default_visual = DefaultVisual (my_Display, screen_number);

  //create_simple_window(display_width,display_height,10,40);
  create_simple_window (1, 1, 800, 600);
  create_gc ();
  XSync (my_Display, False);

  cmap = XCreateColormap (my_Display, my_Window, default_visual, AllocNone);
  return 0;

}

void
refresh ()
{
  XFlush (my_Display);
}

void
close ()
{
  XCloseDisplay (my_Display);
}

void
create_simple_window (int x, int y, unsigned int w, unsigned int h)
{
  Window root_window;
  int window_border_width = 2;
  long fgcolor, bgcolor;
  fgcolor = BlackPixel (my_Display, screen_number);
  bgcolor = WhitePixel (my_Display, screen_number);

  root_window = RootWindow (my_Display, screen_number);
  my_Window = XCreateSimpleWindow (my_Display, root_window, x, y, w, h,
				   window_border_width, bgcolor, fgcolor);
  XMapWindow (my_Display, my_Window);
}

void
create_gc ()
{
  XGCValues values;
  //unsigned long value_mask=0;
  int screen_number = DefaultScreen (my_Display);
  values.foreground = WhitePixel (my_Display, screen_number);
  values.background = BlackPixel (my_Display, screen_number);
  my_GC =
    XCreateGC (my_Display, my_Window, GCForeground | GCBackground, &values);
  if (my_GC < 0)
    {
      printf ("GC Error");
      exit (0);
    }
}


XRectangle *
make_Rectangle (short x, short y, unsigned int width, unsigned int height)
{
  XRectangle *xr = new XRectangle;
  xr->x = x;
  xr->y = y;
  xr->width = width;
  xr->height = height;
  return xr;
}

XColor
make_Color (short r, short g, short b)
{
  XColor *xc = new XColor;
  xc->red = r;
  xc->green = g;
  xc->blue = b;
  XAllocColor (my_Display, cmap, xc);
  return *xc;
}

void
setColor (short r, short g, short b)
{
  fgcolor = make_Color (r, g, b);
  XAllocColor (my_Display, cmap, &fgcolor);
  XSetForeground (my_Display, my_GC, fgcolor.pixel);
}

void
setBackground (short r, short g, short b)
{
  bgcolor = make_Color (r, g, b);
  XAllocColor (my_Display, cmap, &bgcolor);
  XSetBackground (my_Display, my_GC, bgcolor.pixel);
}

XPoint *
make_Point (short x, short y)
{
  XPoint *xp = new XPoint;
  xp->x = x;
  xp->y = y;
  return xp;
}

XArc *
make_Arc (short x, short y, unsigned int width, unsigned int height,
	  short angle1, short angle2)
{
  XArc *xa = new XArc;
  xa->x = x;
  xa->y = y;
  xa->width = width;
  xa->height = height;
  return xa;
}

XSegment *
make_Segment (short x1, short y1, short x2, short y2)
{
  XSegment *xs = new XSegment;
  xs->x1 = x1;
  xs->y1 = y1;
  xs->x2 = x2;
  xs->y2 = y2;
  return xs;
}

XArc *
make_Arc (short x, short y, unsigned short width, unsigned short height,
	  short angle1, short angle2)
{
  XArc *xa = new XArc;
  xa->x = x;
  xa->y = y;
  xa->width = width;
  xa->height = height;
  xa->angle1 = angle1;
  xa->angle2 = angle2;
  return xa;
}

void
drawLine (int x1, int y1, int x2, int y2)
{
  XDrawLine (my_Display, my_Window, my_GC, x1, y1, x2, y2);
}

void
drawTriangle (short x1, short y1, short x2, short y2, short x3, short y3)
{
  int mode;
  XPoint points[] = {
    {x1, y1},
    {x2, y2},
    {x3, y3},
    {x1, y1}
  };
  int npoints = sizeof (points) / sizeof (XPoint);
  XDrawLines (my_Display, my_Window, my_GC, points, npoints, mode =
	      CoordModeOrigin);
}

void
fillTriangle (short x1, short y1, short x2, short y2, short x3, short y3)
{
  //int mode,shape;
  XPoint points[] = {
    {x1, y1},
    {x2, y2},
    {x3, y3},
    {x1, y1}
  };
  //int npoints=sizeof(points)/sizeof(XPoint);

  XFillPolygon (my_Display, my_Window, my_GC, points, 4 /*npoints */ , Convex,
		CoordModeOrigin);
}

void
putPixel (int x, int y)
{
  XDrawPoint (my_Display, my_Window, my_GC, x, y);
  /*XFlush(my_Display);
     sleep(100);
     XCloseDisplay(my_Display); */
}

void
putPixel (XPoint * xp)
{
  XDrawPoint (my_Display, my_Window, my_GC, xp->x, xp->y);
}


void
putPixels (XPoint * points, int npoints, int mode = CoordModeOrigin)
{
  XDrawPoints (my_Display, my_Window, my_GC, points, npoints, mode);
}

//To draw Rectangle
void
fillRectangle (int x, int y, unsigned width, unsigned int height)
{
  XFillRectangle (my_Display, my_Window, my_GC, x, y, width, height);
}

void
drawRectangle (int x, int y, unsigned int width, unsigned int height)
{
  XDrawRectangle (my_Display, my_Window, my_GC, x, y, width, height);
}

//To Draw Ellipse
void
fillEllipse (int x, int y, unsigned int xradius, unsigned int yradius)
{
  XFillArc (my_Display, my_Window, my_GC, x - xradius, y - yradius,
	    2 * xradius, 2 * yradius, 0, 360 * 64);
}

void
drawEllipse (int x, int y, unsigned int xradius, unsigned int yradius)
{
  XDrawArc (my_Display, my_Window, my_GC, x - xradius, y - yradius,
	    2 * xradius, 2 * yradius, 0, 360 * 64);
}

//To Draw Circle
void
fillCircle (int x, int y, unsigned int radius)
{
  XFillArc (my_Display, my_Window, my_GC, x - radius, y - radius, 2 * radius,
	    2 * radius, 0, 360 * 64);
}

void
drawCircle (int x, int y, unsigned int radius)
{
  XDrawArc (my_Display, my_Window, my_GC, x - radius, y - radius, 2 * radius,
	    2 * radius, 0, 360 * 64);
}

void
clearx ()
{
  XClearWindow (my_Display, my_Window);
}


#endif
