#ifndef __OBJ3D_CPP__

#define __OBJ3D_CPP__

#include "include.cpp"

#include "3Dmathdef.cpp"
#include "3Dfacet.cpp"

class Object3D
{
public:
  list < Point > *PointList;
  list < Triangle > *TriangleList;
  list < Facet > *FacetList;

  Object3D ()
  {
    PointList = NULL;
    TriangleList = NULL;
    FacetList = NULL;
  }

  void draw (int edge = 0, int fill = 1)
  {
    int i = 0;
    for (i = 0; i < FacetList->getCount (); i++)
      {
	(*FacetList)[i]->draw (edge, fill);
      }
  }

  void saveToFile (char *);
  void loadFromFile (char *);
  void printOn (ostream &);
};

void
Object3D::printOn (ostream & o)
{
  int i = 0;
  for (i = 0; i < FacetList->getCount (); i++)
    {
      (*FacetList)[i]->printOn (o);
    }
}

void
Object3D::saveToFile (char *file)
{
  ofstream ofile (file, ios::binary | ios::out);
  if (!ofile)
    return;

  int i = 0, j = 0;
  int cnt = FacetList->getCount ();
  cerr << "\nCount=" << cnt;
  XColor col;
  Point pt;

  for (i = 0; i < cnt; i++)
    {

      for (j = 0; j < 3; j++)
	{
	  pt = *((*FacetList)[i]->tr->pts[j]);
	  ofile.write ((char *) &pt, sizeof (Point));
	}
      col = (*FacetList)[i]->xc;
      ofile.write ((char *) (&col), sizeof (XColor));
    }
  ofile.close ();
}

void
Object3D::loadFromFile (char *file)
{
  int i = 0;
  ifstream ifile (file, ios::binary | ios::in);
  if (!ifile)
    return;

  PointList = new list < Point >;
  TriangleList = new list < Triangle >;
  FacetList = new list < Facet >;

  Point *pt1;
  Point *pt2;
  Point *pt3;
  Triangle *tr;
  Facet *f;
  XColor col;

  i = 0;

  while (!ifile.eof ())
    {
      pt1 = new Point;
      pt2 = new Point;
      pt3 = new Point;
      ifile.read ((char *) pt1, sizeof (Point));
      ifile.read ((char *) pt2, sizeof (Point));
      ifile.read ((char *) pt3, sizeof (Point));
      ifile.read ((char *) &col, sizeof (XColor));

      pt1 = PointList->insInRear (pt1);
      pt2 = PointList->insInRear (pt2);
      pt3 = PointList->insInRear (pt3);


      tr = new Triangle (pt1, pt2, pt3);

      tr = TriangleList->insInRear (tr);

      f = new Facet (tr, col);

      f = FacetList->insInRear (f);
    }
  ifile.close ();
}


#endif
